@@ -133,12 +133,12 @@ int CControlSocket::ResetOperation(int nErrorCode)
 	std::unique_ptr<COpData> oldOperation;
 	if (!operations_.empty()) {
 		if (operations_.back()->holdsLock_) {
-			UnlockCache();
+			ReleaseLock();
 		}
 		oldOperation = std::move(operations_.back());
 		operations_.pop_back();
 
-		oldOperation->Reset(nErrorCode);
+		nErrorCode = oldOperation->Reset(nErrorCode);
 	}
 	if (!operations_.empty()) {
 		int ret;
@@ -202,10 +202,7 @@ int CControlSocket::ResetOperation(int nErrorCode)
 						LogMessage(MessageType::Debug_Warning, L"currentServer_ is empty");
 					}
 					else {
-						bool updated = engine_.GetDirectoryCache().UpdateFile(currentServer_, data.remotePath_, data.remoteFile_, true, CDirectoryCache::file, (nErrorCode == FZ_REPLY_OK) ? data.localFileSize_ : -1);
-						if (updated) {
-							SendDirectoryListingNotification(data.remotePath_, false, false);
-						}
+						UpdateCache(data, data.remotePath_, data.remoteFile_, (nErrorCode == FZ_REPLY_OK) ? data.localFileSize_ : -1);
 					}
 				}
 				LogTransferResultMessage(nErrorCode, &data);
@@ -231,6 +228,14 @@ int CControlSocket::ResetOperation(int nErrorCode)
 	return engine_.ResetOperation(nErrorCode);
 }
 
+void CControlSocket::UpdateCache(COpData const &, CServerPath const& serverPath, std::wstring const& remoteFile, int64_t fileSize)
+{
+	bool updated = engine_.GetDirectoryCache().UpdateFile(currentServer_, serverPath, remoteFile, true, CDirectoryCache::file, fileSize);
+	if (updated) {
+		SendDirectoryListingNotification(serverPath, false, false);
+	}
+}
+
 int CControlSocket::DoClose(int nErrorCode)
 {
 	LogMessage(MessageType::Debug_Debug, L"CControlSocket::DoClose(%d)", nErrorCode);
@@ -627,7 +632,7 @@ const std::list<CControlSocket::t_lockInfo>::iterator CControlSocket::GetLockSta
 	return iter;
 }
 
-bool CControlSocket::TryLockCache(locking_reason reason, CServerPath const& directory)
+bool CControlSocket::TryLock(locking_reason reason, CServerPath const& directory)
 {
 	assert(currentServer_);
 	assert(!operations_.empty());
@@ -708,7 +713,7 @@ bool CControlSocket::IsLocked(locking_reason reason, CServerPath const& director
 	return false;
 }
 
-void CControlSocket::UnlockCache()
+void CControlSocket::ReleaseLock()
 {
 	if (operations_.empty() || !operations_.back()->holdsLock_) {
 		return;
@@ -736,12 +741,12 @@ void CControlSocket::UnlockCache()
 
 	// Find other instance waiting for the lock
 	if (!currentServer_) {
-		LogMessage(MessageType::Debug_Warning, L"UnlockCache called with !currentServer_");
+		LogMessage(MessageType::Debug_Warning, L"ReleaseLock called with !currentServer_");
 		return;
 	}
 	for (auto & lockInfo : m_lockInfoList) {
 		if (!lockInfo.pControlSocket->currentServer_) {
-			LogMessage(MessageType::Debug_Warning, L"UnlockCache found other instance with !currentServer_");
+			LogMessage(MessageType::Debug_Warning, L"ReleaseLock found other instance with !currentServer_");
 			continue;
 		}
 
@@ -763,19 +768,19 @@ void CControlSocket::UnlockCache()
 	}
 }
 
-CControlSocket::locking_reason CControlSocket::ObtainLockFromEvent()
+locking_reason CControlSocket::ObtainLockFromEvent()
 {
 	if (operations_.empty()) {
-		return lock_unknown;
+		return locking_reason::unknown;
 	}
 
 	std::list<t_lockInfo>::iterator own = GetLockStatus();
 	if (own == m_lockInfoList.end()) {
-		return lock_unknown;
+		return locking_reason::unknown;
 	}
 
 	if (!own->waiting) {
-		return lock_unknown;
+		return locking_reason::unknown;
 	}
 
 	for (auto iter = m_lockInfoList.cbegin(); iter != own; ++iter) {
@@ -792,7 +797,7 @@ CControlSocket::locking_reason CControlSocket::ObtainLockFromEvent()
 		}
 
 		// Another instance comes before us
-		return lock_unknown;
+		return locking_reason::unknown;
 	}
 
 	own->waiting = false;
@@ -803,20 +808,21 @@ CControlSocket::locking_reason CControlSocket::ObtainLockFromEvent()
 
 void CControlSocket::OnObtainLock()
 {
-	if (ObtainLockFromEvent() == lock_unknown) {
+	if (ObtainLockFromEvent() == locking_reason::unknown) {
 		return;
 	}
 
 	SendNextCommand();
 
-	UnlockCache();
+	ReleaseLock();
 }
 
 bool CControlSocket::IsWaitingForLock()
 {
 	std::list<t_lockInfo>::iterator own = GetLockStatus();
-	if (own == m_lockInfoList.end())
+	if (own == m_lockInfoList.end()) {
 		return false;
+	}
 
 	return own->waiting == true;
 }
