@@ -7,6 +7,8 @@
 // Since COptionsBase is virtual, the user of the engine has to create a
 // derived class which handles settings-reading and writing.
 
+#include <memory>
+
 enum engineOptions
 {
 	OPTION_USEPASV,			// Use passive mode unless overridden by
@@ -84,15 +86,21 @@ enum engineOptions
 	OPTIONS_ENGINE_NUM
 };
 
+namespace pugi {
+class xml_document;
+}
+
 class COptionsBase
 {
 public:
 	inline virtual ~COptionsBase() {};
 	virtual int GetOptionVal(unsigned int nID) = 0;
 	virtual std::wstring GetOption(unsigned int nID) = 0;
+	virtual std::unique_ptr<pugi::xml_document> GetOptionXml(unsigned int nID) = 0;
 
 	virtual bool SetOption(unsigned int nID, int value) = 0;
 	virtual bool SetOption(unsigned int nID, std::wstring const& value) = 0;
+	virtual bool SetOptionXml(unsigned int nID, std::unique_ptr<pugi::xml_document> const& value) = 0;
 };
 
 #endif
