@@ -9,6 +9,7 @@ class CLocalListViewDropTarget;
 #ifdef __WXMSW__
 class CVolumeDescriptionEnumeratorThread;
 #endif
+class CView;
 class CWindowTinter;
 
 class CLocalFileData final : public CGenericFileData
@@ -16,7 +17,7 @@ class CLocalFileData final : public CGenericFileData
 public:
 	std::wstring name;
 #ifdef __WXMSW__
-	fz::sparse_optional<wxString> label;
+	fz::sparse_optional<std::wstring> label;
 #endif
 	fz::datetime time;
 	int64_t size;
@@ -31,7 +32,7 @@ class CLocalListView final : public CFileListCtrl<CLocalFileData>, CStateEventHa
 	friend class CLocalListViewSortType;
 
 public:
-	CLocalListView(wxWindow* parent, CState& state, CQueueView *pQueue);
+	CLocalListView(CView* parent, CState& state, CQueueView *pQueue);
 	virtual ~CLocalListView();
 
 protected:
@@ -74,7 +75,7 @@ protected:
 	bool IsItemValid(unsigned int item) const;
 	CLocalFileData *GetData(unsigned int item);
 
-	virtual CSortComparisonObject GetSortComparisonObject();
+	virtual std::unique_ptr<CFileListCtrlSortBase> GetSortComparisonObject() override;
 
 	void RefreshFile(const wxString& file);
 
@@ -85,12 +86,14 @@ protected:
 
 	CLocalPath m_dir;
 
-	int m_dropTarget;
+	int m_dropTarget{-1};
 
 	wxString MenuMkdir();
 
 	std::unique_ptr<CWindowTinter> m_windowTinter;
 
+	CView *m_parentView{};
+
 	// Event handlers
 	DECLARE_EVENT_TABLE()
 	void OnItemActivated(wxListEvent& event);
@@ -105,11 +108,12 @@ protected:
 	void OnMenuOpen(wxCommandEvent& event);
 	void OnMenuEdit(wxCommandEvent& event);
 	void OnMenuEnter(wxCommandEvent& event);
+	void OnMenuRefresh(wxCommandEvent& event);
+
 #ifdef __WXMSW__
 	void OnVolumesEnumerated(wxCommandEvent& event);
-	CVolumeDescriptionEnumeratorThread* m_pVolumeEnumeratorThread;
+	std::unique_ptr<CVolumeDescriptionEnumeratorThread> volumeEnumeratorThread_;
 #endif
-	void OnMenuRefresh(wxCommandEvent& event);
 };
 
 #endif
