@@ -23,7 +23,8 @@ COptions* COptions::m_theOptions = 0;
 enum Type
 {
 	string,
-	number
+	number,
+	xml
 };
 
 enum Flags
@@ -36,7 +37,7 @@ enum Flags
 
 struct t_Option
 {
-	const char name[30];
+	const char name[40];
 	const Type type;
 	const std::wstring defaultValue; // Default values are stored as string even for numerical options
 	const Flags flags; // internal items won't get written to settings file nor loaded from there
@@ -226,10 +227,17 @@ t_OptionsCache& t_OptionsCache::operator=(int v)
 	return *this;
 }
 
+t_OptionsCache& t_OptionsCache::operator=(std::unique_ptr<pugi::xml_document> const& v)
+{
+	xmlValue = std::make_unique<pugi::xml_document>();
+	xmlValue->append_copy(v->first_child());
+
+	return *this;
+}
+
 COptions::COptions()
 {
 	m_theOptions = this;
-	m_pXmlFile = 0;
 
 	SetDefaultValues();
 
@@ -241,12 +249,11 @@ COptions::COptions()
 	CLocalPath const dir = InitSettingsDir();
 
 	CInterProcessMutex mutex(MUTEX_OPTIONS);
-	m_pXmlFile = new CXmlFile(dir.GetPath() + _T("filezilla.xml"));
-	if (!m_pXmlFile->Load()) {
-		wxString msg = m_pXmlFile->GetError() + _T("\n\n") + _("For this session the default settings will be used. Any changes to the settings will not be saved.");
+	xmlFile_ = std::make_unique<CXmlFile>(dir.GetPath() + _T("filezilla.xml"));
+	if (!xmlFile_->Load()) {
+		wxString msg = xmlFile_->GetError() + _T("\n\n") + _("For this session the default settings will be used. Any changes to the settings will not be saved.");
 		wxMessageBoxEx(msg, _("Error loading xml file"), wxICON_ERROR);
-		delete m_pXmlFile;
-		m_pXmlFile = 0;
+		xmlFile_.reset();
 	}
 	else {
 		CreateSettingsXmlElement();
@@ -269,8 +276,6 @@ std::map<std::string, unsigned int> COptions::GetNameOptionMap() const
 COptions::~COptions()
 {
 	COptionChangeEventHandler::UnregisterAllHandlers();
-
-	delete m_pXmlFile;
 }
 
 int COptions::GetOptionVal(unsigned int nID)
@@ -293,6 +298,18 @@ std::wstring COptions::GetOption(unsigned int nID)
 	return m_optionsCache[nID].strValue;
 }
 
+std::unique_ptr<pugi::xml_document> COptions::GetOptionXml(unsigned int nID)
+{
+	if (nID >= OPTIONS_NUM) {
+		return nullptr;
+	}
+
+	auto value = std::make_unique<pugi::xml_document>();
+	value->append_copy(m_optionsCache[nID].xmlValue->first_child());
+
+	return std::move(value);
+}
+
 bool COptions::SetOption(unsigned int nID, int value)
 {
 	if (nID >= OPTIONS_NUM) {
@@ -321,6 +338,24 @@ bool COptions::SetOption(unsigned int nID, std::wstring const& value)
 	return true;
 }
 
+bool COptions::SetOptionXml(unsigned int nID, std::unique_ptr<pugi::xml_document> const& value)
+{
+	if (nID >= OPTIONS_NUM) {
+		return false;
+	}
+
+	if (options[nID].type != xml) {
+		return false;
+	}
+
+	auto doc = std::make_unique<pugi::xml_document>();
+	doc->append_copy(value->first_child());
+
+	ContinueSetOption(nID, doc);
+
+	return true;
+}
+
 template<typename T>
 void COptions::ContinueSetOption(unsigned int nID, T const& value)
 {
@@ -374,11 +409,11 @@ bool COptions::OptionFromFzDefaultsXml(unsigned int nID)
 
 pugi::xml_node COptions::CreateSettingsXmlElement()
 {
-	if (!m_pXmlFile) {
+	if (!xmlFile_) {
 		return pugi::xml_node();
 	}
 
-	auto element = m_pXmlFile->GetElement();
+	auto element = xmlFile_->GetElement();
 	if (!element) {
 		return element;
 	}
@@ -393,6 +428,9 @@ pugi::xml_node COptions::CreateSettingsXmlElement()
 		if (options[i].type == string) {
 			SetXmlValue(i, GetOption(i));
 		}
+		else if (options[i].type == xml) {
+			SetXmlValue(i, GetOptionXml(i));
+		}
 		else {
 			SetXmlValue(i, GetOptionVal(i));
 		}
@@ -408,7 +446,7 @@ void COptions::SetXmlValue(unsigned int nID, int value)
 
 void COptions::SetXmlValue(unsigned int nID, std::wstring const& value)
 {
-	if (!m_pXmlFile) {
+	if (!xmlFile_) {
 		return;
 	}
 
@@ -435,6 +473,34 @@ void COptions::SetXmlValue(unsigned int nID, std::wstring const& value)
 	}
 }
 
+void COptions::SetXmlValue(unsigned int nID, std::unique_ptr<pugi::xml_document> const& value)
+{
+	if (!xmlFile_) {
+		return;
+	}
+
+	auto settings = CreateSettingsXmlElement();
+	if (settings) {
+		pugi::xml_node setting;
+		for (setting = settings.child("Setting"); setting; setting = setting.next_sibling("Setting")) {
+			const char *attribute = setting.attribute("name").value();
+			if (!attribute) {
+				continue;
+			}
+			if (!strcmp(attribute, options[nID].name)) {
+				break;
+			}
+		}
+		if (setting) {
+			settings.remove_child(setting);
+		}
+		setting = settings.append_child("Setting");
+		SetTextAttribute(setting, "name", options[nID].name);
+
+		setting.append_copy(value->first_child());
+	}
+}
+
 int COptions::Validate(unsigned int nID, int value)
 {
 	switch (nID)
@@ -555,9 +621,17 @@ std::wstring COptions::Validate(unsigned int nID, std::wstring const& value)
 	return value;
 }
 
+std::unique_ptr<pugi::xml_document> COptions::Validate(unsigned int, std::unique_ptr<pugi::xml_document> const& value)
+{
+	auto res = std::make_unique<pugi::xml_document>();
+	res->append_copy(value->first_child());
+
+	return res;
+}
+
 void COptions::SetServer(std::wstring path, ServerWithCredentials const& server)
 {
-	if (!m_pXmlFile) {
+	if (!xmlFile_) {
 		return;
 	}
 
@@ -565,7 +639,7 @@ void COptions::SetServer(std::wstring path, ServerWithCredentials const& server)
 		return;
 	}
 
-	auto element = m_pXmlFile->GetElement();
+	auto element = xmlFile_->GetElement();
 
 	while (!path.empty()) {
 		std::wstring sub;
@@ -596,7 +670,7 @@ void COptions::SetServer(std::wstring path, ServerWithCredentials const& server)
 	}
 
 	CInterProcessMutex mutex(MUTEX_OPTIONS);
-	m_pXmlFile->Save(true);
+	xmlFile_->Save(true);
 }
 
 bool COptions::GetServer(std::wstring path, ServerWithCredentials& server)
@@ -605,10 +679,10 @@ bool COptions::GetServer(std::wstring path, ServerWithCredentials& server)
 		return false;
 	}
 
-	if (!m_pXmlFile) {
+	if (!xmlFile_) {
 		return false;
 	}
-	auto element = m_pXmlFile->GetElement();
+	auto element = xmlFile_->GetElement();
 
 	while (!path.empty()) {
 		std::wstring sub;
@@ -739,11 +813,18 @@ void COptions::LoadOptionFromElement(pugi::xml_node option, std::map<std::string
 			fz::scoped_lock l(m_sync_);
 			m_optionsCache[iter->second] = numValue;
 		}
-		else {
+		else if (options[iter->second].type == string) {
 			value = Validate(iter->second, value);
 			fz::scoped_lock l(m_sync_);
 			m_optionsCache[iter->second] = value;
 		}
+		else {
+			fz::scoped_lock l(m_sync_);
+			if (!option.first_child().empty()) {
+				m_optionsCache[iter->second].xmlValue = std::make_unique<pugi::xml_document>();
+				m_optionsCache[iter->second].xmlValue->append_copy(option.first_child());
+			}
+		}
 	}
 }
 
@@ -784,12 +865,12 @@ void COptions::Save()
 		return;
 	}
 
-	if (!m_pXmlFile) {
+	if (!xmlFile_) {
 		return;
 	}
 
 	CInterProcessMutex mutex(MUTEX_OPTIONS);
-	m_pXmlFile->Save(true);
+	xmlFile_->Save(true);
 }
 
 void COptions::SaveIfNeeded()
@@ -942,7 +1023,13 @@ void COptions::SetDefaultValues()
 {
 	fz::scoped_lock l(m_sync_);
 	for (int i = 0; i < OPTIONS_NUM; ++i) {
-		m_optionsCache[i] = options[i].defaultValue;
 		m_optionsCache[i].from_default = false;
+		if (options[i].type == xml) {
+			m_optionsCache[i].xmlValue = std::make_unique<pugi::xml_document>();
+			m_optionsCache[i].xmlValue->load_string(fz::to_string(options[i].defaultValue).c_str());
+		}
+		else {
+			m_optionsCache[i] = options[i].defaultValue;
+		}
 	}
 }
