@@ -3,6 +3,7 @@
 #include "context_control.h"
 #include "filelist_statusbar.h"
 #include "filezillaapp.h"
+#include "list_search_panel.h"
 #include "local_recursive_operation.h"
 #include "LocalListView.h"
 #include "LocalTreeView.h"
@@ -228,6 +229,12 @@ void CContextControl::CreateContextControls(CState& state)
 	auto remoteRecursiveStatus = new CRecursiveOperationStatus(context_controls.pRemoteListViewPanel, state, false);
 	context_controls.pRemoteListViewPanel->SetFooter(remoteRecursiveStatus);
 
+	context_controls.pLocalListSearchPanel = new CListSearchPanel(context_controls.pLocalListViewPanel, context_controls.pLocalListView, context_controls.pState, true);
+	context_controls.pLocalListViewPanel->SetSearchPanel(context_controls.pLocalListSearchPanel);
+
+	context_controls.pRemoteListSearchPanel = new CListSearchPanel(context_controls.pRemoteListViewPanel, context_controls.pRemoteListView, context_controls.pState, false);
+	context_controls.pRemoteListViewPanel->SetSearchPanel(context_controls.pRemoteListSearchPanel);
+
 	const int layout = COptions::Get()->GetOptionVal(OPTION_FILEPANE_LAYOUT);
 	const int swap = COptions::Get()->GetOptionVal(OPTION_FILEPANE_SWAP);
 
