@@ -1,5 +1,5 @@
-#ifndef __XH_TEXT_EX_H__
-#define __XH_TEXT_EX_H__
+#ifndef FILEZILLA_INTERFACE_XH_TEXT_EX_HEADER
+#define FILEZILLA_INTERFACE_XH_TEXT_EX_HEADER
 
 #include <wx/xrc/xh_text.h>
 
@@ -7,7 +7,7 @@
 #define wxTextCtrlXmlHandlerEx wxTextCtrlXmlHandler
 #else
 
-class wxTextCtrlXmlHandlerEx : public wxTextCtrlXmlHandler
+class wxTextCtrlXmlHandlerEx final : public wxTextCtrlXmlHandler
 {
 	DECLARE_DYNAMIC_CLASS(wxTextCtrlXmlHandlerEx)
 
@@ -17,4 +17,4 @@ public:
 
 #endif
 
-#endif //__XH_TEXT_EX_H__
+#endif
